import React, { useRef, useState, Suspense, useEffect } from 'react'
import { Box } from '@chakra-ui/core'
import { OrbitControls, Sky } from '@react-three/drei'
import { Canvas, useFrame } from 'react-three-fiber'
import { useSphere } from '@react-three/cannon'

import Plaza from '../models/Plaza'
import PlazaCollision from '../models/PlazaCollision'
import { Physics } from '@react-three/cannon'
import { Event } from 'three'

function Cube() {
  const [ref, api] = useSphere(() => ({
    fixedRotation: true,
    mass: 1,
    position: [0, 20, 0],
  }))
  const velocity = useRef([0, 0, 0])

  const [moveForward, setMoveForward] = useState(false)
  const [moveBackwards, setMoveBackwards] = useState(false)
  const [moveLeft, setMoveLeft] = useState(false)
  const [moveRight, setMoveRight] = useState(false)

  useEffect(() => {
    api.velocity.subscribe((v) => (velocity.current = v))

    const onKeyDown = (event: Event) => {
      switch (event.keyCode) {
        case 38: // up
        case 87: // w
          setMoveForward(true)
          break

        case 37: // left
        case 65: // a
          setMoveLeft(true)
          break

        case 40: // down
        case 83: // s
          setMoveBackwards(true)
          break

        case 39: // right
        case 68: // d
          setMoveRight(true)
          break
      }
    }

    const onKeyUp = (event: Event) => {
      switch (event.keyCode) {
        case 38: // up
        case 87: // w
          setMoveForward(false)
          break

        case 37: // left
        case 65: // a
          setMoveLeft(false)
          break

        case 40: // down
        case 83: // s
          setMoveBackwards(false)
          break

        case 39: // right
        case 68: // d
          setMoveRight(false)
          break
      }
    }

    document.addEventListener('keydown', onKeyDown, false)
    document.addEventListener('keyup', onKeyUp, false)
  }, [])

  useFrame(() => {
    if (api && ref.current) {
      let x, z
      x = z = 0

      if (moveForward) {
        z += -5
      }
      if (moveBackwards) {
        z += 5
      }
      if (moveLeft) {
        x += -5
      }
      if (moveRight) {
        x += 5
      }

      console.log(x, z)

      if (x != 0 || z != 0) {
        api.velocity.set(x, velocity.current[1], z)
      }
    }
  })

  return (
    <mesh ref={ref}>
      <boxBufferGeometry attach="geometry" />
    </mesh>
  )
}

const Museo: React.FC = () => {
  return (
    <Box w="100vw" h="100vh">
      <Canvas>
        <ambientLight />
        <pointLight position={[10, 10, 10]} />
        <Suspense fallback={null}>
          <Plaza />
          <Physics>
            <PlazaCollision />
            <Cube />
          </Physics>
        </Suspense>
        <OrbitControls />
        <Sky
          distance={550000}
          azimuth={0.25}
          mieDirectionalG={0.83}
          mieCoefficient={0.008}
          inclination={0.5}
          turbidity={8}
          rayleigh={2.9}
        />
      </Canvas>
    </Box>
  )
}

export default Museo
